/* this example is compatible with Biddy v1.8.2 */
/* you have to use C++ compiler with C++11 support */
/* Linux + gcc: compile with g++ -DUNIX -std=c++11 -o program.exe source.c -I. -L. -lbiddy -lgmp */
/* Windows + MINGW: compile with g++ -DMINGW -std=c++11 -o program.exe source.c -I. -L. -lbiddy -lgmp */
#include <iostream>
#include <iomanip>
#include <string>
#include <array>

#include "biddy.h"

/* these macros are helpful, e.g. they enable usage of C++ strings */
#define AddVariable() Biddy_GetVariableEdge(Biddy_AddVariable())
#define AddVariableByName(x) Biddy_GetVariableEdge(Biddy_AddVariableByName((Biddy_String)(((std::string)x).c_str())))
#define AddTmpFormula(name,f) Biddy_AddTmpFormula((Biddy_String)(((std::string)name).c_str()),f);
#define FindFormula(name,idx,f) Biddy_FindFormula((Biddy_String)(((std::string)name).c_str()),idx,f);
#define Str2Var(x) (Biddy_GetVariable((Biddy_String)x))
int main() {
  std::cout << "DEMO\n";
  Biddy_InitAnonymous(BIDDYTYPEOBDDC); /* use robdds with complemented edges */
  /* these variables are used globally */
  Biddy_Edge f,g;
  unsigned int varnum;
  unsigned int idx;

  /* TEST 1 - CONSTANTS */
  if (Biddy_GetConstantZero() == Biddy_Not(Biddy_GetConstantOne())) {
    std::cout << "CONSTANTS OK\n";
  } else {
    std::cout << "CONSTANTS ERROR\n";
  }

  /* TEST 2 - VARIABLES AND BASIC OPERATIONS */
  f = AddVariableByName("x1");
  g = AddVariableByName("x2");
  f = Biddy_And(f,g);
  g = Biddy_Not(Biddy_ITE(f,Biddy_Not(g),Biddy_GetConstantOne()));
  if ((Biddy_GetTopVariable(f) == Str2Var("x1")) && (f == g)) {
    std::cout << "VARIABLES AND BASIC OPERATIONS OK\n";
  } else {
    std::cout << "VARIABLES AND BASIC OPERATIONS ERROR\n";
  }

  /* TEST 3 - PREFIX INPUT AND FORMULAE */
  varnum = 4; /* number of variables */
  unsigned int num = 13; /* generate 13th minterm */
  std::string *variableName = new std::string[varnum];
  for (unsigned int i = 0; i < varnum; i++) {
    variableName[i] = "a" + std::to_string(i); /* you need C++11 for this */
  }
  AddTmpFormula("m13",Biddy_GetConstantOne());
  AddTmpFormula("ithv",NULL); /* ithv will be used to store ith variable */
  for (unsigned int i = 0; i < varnum; i++,num/=2) {
    AddTmpFormula("ithv",AddVariableByName(variableName[i]));
    if (num % 2) {
      AddTmpFormula("m13",Biddy_Eval1((Biddy_String)"(AND m13 ithv)"));
    } else {
      AddTmpFormula("m13",Biddy_Eval1((Biddy_String)"(AND m13 (NOT ithv))"));
    }
  }
  delete[] variableName;
  FindFormula("m13",&idx,&f);
  g = Biddy_Eval1((Biddy_String)"(AND a3 a2 (NOT a1) a0)"); /* 13th minterm */
  if (f == g) {
    std::cout << "PREFIX INPUT AND FORMULAE OK\n";
  } else {
    std::cout << "PREFIX INPUT AND FORMULAE ERROR\n";
  }

  /* TEST 4 - INFIX INPUT AND TABLE OUTPUT */
  /* we are using functions for outputs of one-bit full adder */
  f = Biddy_Eval2((Biddy_String)"~x&~y&c|~x&y&~c|x&~y&~c|x&y&c"); /* sum */
  g = Biddy_Eval2((Biddy_String)"x&y|x&c|y&c"); /* cout */
  std::cout << "Truth table for function f (should be 0110 1001)\n";
  Biddy_PrintfTable(f);
  std::cout << "Truth table for function g (should be 0001 0111)\n";
  Biddy_PrintfTable(g);

  /* TEST 5 - NUMBERED (NONAME) VARIABLES AND STATISTICAL INFO */
  varnum = 32; /* generate 32 variables */
  f = Biddy_GetConstantZero();
  g = Biddy_GetConstantOne();
  for (unsigned int i = 0; i < varnum; i++) {
    Biddy_Edge v = AddVariable();
    f = Biddy_Or(v,f);
    g = Biddy_ITE(v,Biddy_GetConstantZero(),g);
  }
  if (f == Biddy_Not(g)) {
    std::cout << "NUMBERED VARIABLES OK\n";
  } else {
    std::cout << "NUMBERED VARIABLES ERROR\n";
  }
  std::cout.precision(0); /* used for reporting number of minterms */
  std::cout << "Function f depends on (should be 32): " << Biddy_DependentVariableNumber(f,FALSE) << " variables.\n";
  std::cout << "Function f has (should be 4294967295): " << std::fixed << Biddy_CountMinterms(f,0) << " minterms.\n";
  std::cout << "BDD for function f has (should be 33): " << Biddy_CountNodes(f) << " nodes.\n";
  std::cout << "Function g depends on (should be 32): " << Biddy_DependentVariableNumber(g,FALSE) << " variables.\n";
  std::cout << "Function g has (should be 1): " << Biddy_CountMinterms(g,0) << " minterms.\n";
  std::cout << "BDD for function g has (should be 33): " << Biddy_CountNodes(g) << " nodes.\n";

  /* TEST 6 - RAW INPUT, RAW OUTPUT, VARIABLE SWAPPING, AND EXPORT TO DOT */
  /* (for raw input, all variables should already exists in the correct ordering) */
  std::cout << "RAW INPUT, RAW OUTPUT, VARIABLE SWAPPING, AND EXPORT TO DOT\n";
  AddVariableByName("B");
  AddVariableByName("i");
  AddVariableByName("d");
  AddVariableByName("y");
  std::string bddName;
  std::string bddData = (std::string)
    "Biddy" + /* name */
    " " + /* at least one space */
    "B" + /* top variable */
    "(*i(d(1)(y(*1)(1)))(d(y(*1)(1))(1)))" + /* 'else' successor */
    "(i(d(1)(y(*1)(1)))(d(y(*1)(1))(1)))"; /* 'then' successor */
  bddName = Biddy_Eval0((Biddy_String)(bddData.c_str()));
  FindFormula(bddName,&idx,&f);
  while (Biddy_IsSmaller(Str2Var("y"),Str2Var("d"))) { /* smaller = topmore */
    Biddy_SwapWithHigher(Str2Var("y")); /* higher = bottommore */
  }
  std::cout << "Function " << bddName << std::endl;
  std::cout << bddName << " ";
  Biddy_PrintfBDD(f);
  std::cout << "Correct raw output is" << std::endl << bddData << std::endl;
  Biddy_WriteDot("biddy.dot",f,bddName.c_str(),-1,FALSE);
  std::cout << "USE 'dot -y -Tpng -O biddy.dot' TO VISUALIZE GENERATED BDD!\n";

  /* TEST 7 - EXPORT TO BDDVIEW FORMAT */
  Biddy_WriteBddview("biddyexample.bddview",f,"Biddy",NULL);
  printf("Use BDD Scout to visualize generated biddyexample.bddview.\n");

  Biddy_Exit();
}
